// #include <cstdio>
#include <malloc.h>
#include <cstdlib>
#include <cassert>
// #include <mpi.h>
#include "mg.h"
#include "header.h"

int AllocLevels(const int nlevels, const int nx, const int ny, PointerStruct* pp)
{
 const double dzero=0.0;
 const int     zero = 0;
 int    ierr, mx, my, nnode, sksize, lev;

 ierr = (nlevels>MAXLEVEL) ? -11 : 0 ;
 if (ierr<0) return ierr; 

 for (lev=0; lev <MAXLEVEL; lev++)
 {
   pp->nx[lev] = pp->ny[lev] = 0;
   pp->ik[lev] = pp->id[lev] = NULL;  
   pp->sk[lev] = pp->dd[lev] = pp->f[lev]  = pp->u[lev] = pp->aux[lev] = NULL;
 }
 
 pp->nx[0] = nx;
 pp->ny[0] = ny;
 for (lev=1; lev<nlevels; lev++) 
 {
   pp->nx[lev] = 2* pp->nx[lev-1];
   pp->ny[lev] = 2* pp->ny[lev-1];
 }
 
 for (lev=0; lev<nlevels; lev++) 
 {
   mx     = pp->nx[lev];
   my     = pp->ny[lev];
   nnode  = (mx+1) * (my+1);		    /* number of degrees of freedom  */
//	printf("%u  %u\n",my,nnode);

   sksize = 4*mx*my+2*(mx+my)+1;			/* symm. storage     */
   pp->sk[lev] = (double *)malloc(sksize*sizeof(dzero));/* K                 */
   pp->ik[lev] = (int *) malloc(sksize*sizeof(zero));	/* crs pointer for K */
   pp->id[lev] = (int *) malloc(nnode*sizeof(zero));	/* crs pointer for K */
   pp->u[lev]  = (double *) malloc(nnode*sizeof(dzero));/* temperature       */
   pp->f[lev]  = (double *) malloc(nnode*sizeof(dzero));/* r.h.s.  	     */
   pp->dd[lev] = (double *) malloc(nnode*sizeof(dzero));/* diagonale         */
   pp->aux[lev]= (double *) malloc(nnode*sizeof(dzero));/* temp. vector      */   
   if (pp->sk[lev]==NULL || pp->ik[lev]==NULL || pp->id[lev]==NULL || 
       pp->u[lev] ==NULL || pp->f[lev] ==NULL || pp->dd[lev]==NULL || 
                                                pp->aux[lev]==NULL   ) ierr=1;
 }
 return ierr;
}

int IniLevels(const double xl, const double xr, const double yb, const double yt, 
	      const int neigh[],
// 	      const int color, const int , const MPI::Intracomm& icomm, 
	      const int nlevels, const PointerStruct *pp)
{
 const double dzero=0.0;
 double *sk, *u, *f, *dd;
 int    *ik, *id, nx, ny, nnode, lev, ierr;

 ierr = 0;
 
 for (lev=0; lev<nlevels; lev++) 
 {
   nx    = pp->nx[lev];
   ny    = pp->ny[lev];
   nnode = (nx+1) * (ny+1);
   u     = pp->u[lev];

   if (lev==nlevels-1) 
     { SetU(nx, ny, u);			/* Dirichlet B.C. on fine grid */
     }
   else 
     { vdcopy(nnode,u,1,&dzero,0);	/* Hom. Dirichlet B.C. for correction */
     }
 }
 
 for (lev=0; lev<nlevels; lev++) 
 {
   nx = pp->nx[lev];
   ny = pp->ny[lev];   
   u  = pp->u[lev];
   f  = pp->f[lev];
   sk = pp->sk[lev];
   ik = pp->ik[lev];
   id = pp->id[lev];
   dd = pp->dd[lev];
      
   GetMatrix(nx, ny, xl, xr, yb, yt, sk, id, ik, f); /* generate K, rhs   */

   ApplyDirichletBC1(nx, ny, neigh, u, sk, id, ik, f); /* apply B.C.       */

   GetDiag(nx, ny, sk, id, dd);

/*  
   printf("\nLevel %u :\n",lev+1);     
   DebugMatrix((nx+1)*(ny+1), sk, id, ik); 
   DebugD((nx+1)*(ny+1),f); 
*/
 } 
 return ierr;
}


int FreeLevels(const int nlevels, PointerStruct* pp)
{
 int lev, ierr = 0;

 free(pp->csk);
 free(pp->cik);
 free(pp->cid);
 free(pp->loc2glob);

/* for (lev=nlevels-1; lev>=0; lev--)  */
 for (lev=0; lev<=nlevels-1; lev++) 
 {
   free(pp->aux[lev]);
   free( pp->dd[lev]);
   free(  pp->f[lev]);
   free(  pp->u[lev]);
   free( pp->id[lev]);
   free( pp->ik[lev]);
   free( pp->sk[lev]);
 }
 
 return ierr;
}
