#include "header.h"
/*
	Computes the rows (iza..ize) of w := w + alfa * sk * u   iff alfa >0
		                       w := sk * u              iff alfa ==0

	IN:	izf	first row
		izl	last row
		u	vector
		id, ik	crs pointers of sk
		sk	symmetric matrix
		alfa	coefficient
	I/O	w	vector
*/
		 
 
void  CrsMult(const int iza, const int ize, double w[], const double u[], 
	const int id[], const int ik[], const double sk[], const double& alfa)
{
    double fak;   
    if (alfa == 0.0) 
    {
	const int n = ize - iza +1;
	const double dzero=0.0;
	vdcopy(n, &w[iza], 1, &dzero, 0);
	fak = 1.0;
    } 
    else 
    {
        fak = alfa;
    }

    
    double ui,wi;
    int ij1, ij2 = (iza == 0) ? -1 : id[iza-1];
    
    for (int i = iza; i <= ize; ++i) 
    {
      ij1 = ij2 + 1;
      ij2 = id[i];
      ui  = u[i];
      wi  = sk[ij1] * ui;
      ui *= fak;
      for (int ij = ij1+1; ij <=ij2; ++ij) 
      {
	const int &j = ik[ij];
	if (j <= ize) 
        {
	   wi   += sk[ij] * u[j]; 
	   w[j] += sk[ij] * ui; 
 	 }
      }	
      w[i] += fak * wi;
    }
    return;
}


void GetDiag(const int nx, const int ny, const double sk[], const int id[], double d[])
{
    const int nd = (nx+1)*(ny+1);
          int ia = 0;
    for (int i=0; i < nd; ++i)
    {
       d[i] = sk[ia];
       ia   = id[i]+1;
    }
    return;
}

