/*
   (0,1) * (0,1) is divided into procx * procy subdomains 
   numbered row by row,
   according to myid, the array neigh contains the id's
   of the neighbors at South, East, North, West
   
	IN:  	myid	  process id
		procx	  number of subdomains in x-direction
		procy	  number of subdomains in x-direction
	OUT     neigh     id's of neighbors
		color     black/white coloring (check board) of subdomains   	
*/


void IniGeom(const int myid, const int procx, const int procy, int neigh[], int& color)
{ 
 const int ky = myid/procx;
 const int kx = myid%procx;	/*    MOD(myid,procx)    */

 neigh[0] = (ky == 0)       ?  -1 : myid-procx;		/*   South   */
 neigh[1] = (kx == procx-1) ?  -1 : myid+1;		/*   East    */
 neigh[2] = (ky == procy-1) ?  -1 : myid+procx;		/*   North   */
 neigh[3] = (kx == 0)       ?  -1 : myid-1;		/*   West    */

 color = (kx+ky) & 1 ;
 return;
}

/* 
   (0,1) * (0,1) is divided into procx * procy subdomains 
   numbered row by row,
   according to myid, the coordinates of the lower left corner
   (xl,yb) and the upper right corner (xr,yt) are generated

	IN:  	myid	  process id
		procx	  number of subdomains in x-direction
		procy	  number of subdomains in x-direction
	OUT:    (xl,yb)   coordinates of the lower left corner
		(xr,yt)   coordinates the upper right corner
*/

void IniCoord(const int myid, const int procx, const int procy, 
	      double& xl, double& xr, double& yb, double& yt)
{
    const int kx = myid % procx;
    const int ky = myid / procx;
    const double hx = 1. / procx;
    const double hy = 1. / procy;
					/* subdomain - part of unit square */
    xl = kx * hx;                      /* left    */
    xr = (kx + 1) * hx;                /*  right  */
    yb = ky * hy;                      /*  bottom */
    yt = (ky + 1) * hy;                /*  top */

    return;
} 

