//#include <cmath>
// #include <stdio.h>
#include "header.h"

/*  Bilinear interpolation and restriction	*/

/*  SPEC.: topological rectangular grid		*/

void Interpolate(const int nx, const int ny, double w[],  
		  const int nx_c, const int ny_c, const double w_c[], const int neigh[])
{
 const double dzero=0.0;
 int i_c,j_c,k,k_c,k_m,l;

 k   = 0;
 k_m = 0;
 for (j_c = 0;  j_c < ny_c;  j_c++)
 {
   k_c = k_m;
   for (i_c = 0;  i_c < nx_c;  i_c++)
   {
     w[k] = w_c[k_c];
     k++;
     w[k] = 0.5*(w_c[k_c]+w_c[k_c+1]);
     k++; k_c++;
   }
   w[k] = w_c[k_c];					/* right column	*/
   k++; k_c++;

   k_c = k_m; 
   for (i_c = 0;  i_c < nx_c;  i_c++)
   {
     w[k] = 0.5*(w_c[k_c]+w_c[k_c+nx_c+1]);
     k++;
     w[k] = 0.25*(w_c[k_c]+w_c[k_c+nx_c+1]+w_c[k_c+1]+w_c[k_c+nx_c+2]);
     k++; k_c++;
   }
   w[k] = 0.5*(w_c[k_c]+w_c[k_c+nx_c+1]);			/* right column	*/
   k++; k_c++;
   k_m += nx_c+1;
 } 
 
   k_c = k_m;
   for (i_c = 0;  i_c < nx_c;  i_c++)			/* top row	*/
   {
     w[k] = w_c[k_c];
     k++;
     w[k] = 0.5*(w_c[k_c]+w_c[k_c+1]);
     k++; k_c++;
   }
   w[k] = w_c[k_c];					/* top right vertex */

	/* Handle Dirichlet B.C.  = neigh[.] == -1 */
 
 for (l=0; l<=3; l++)
 {
  if (neigh[l]==-1)
    switch (l)
    {
     case 0 : 
     {
       vdcopy(nx+1, &w[0], 1, &dzero, 0);
       break;
     }
     case 2 : 
     {
       vdcopy(nx+1, &w[ny*(nx+1)], 1, &dzero, 0);
       break;
     }
     case 1 : 
     {
       vdcopy(ny+1, &w[nx], nx+1, &dzero, 0);
       break;
     }     
     case 3 : 
     {
       vdcopy(ny+1, &w[0], nx+1, &dzero, 0);
       break;
     }     
    }
 }

 return;
}



void Restrict(const int nx, const int ny, const double w[],  
	      const int nx_c, const int ny_c, double w_c[], const int neigh[])
{
 const double dzero=0.0;
 int i_c,j_c,k,k_c,k_m,l;

 vdcopy((nx_c+1)*(ny_c+1), w_c, 1, &dzero, 0);		/* w_c := 0	*/

 k   = 0;
 k_m = 0;
 for (j_c = 0;  j_c < ny_c;  j_c++)
 {
   k_c = k_m;
   for (i_c = 0;  i_c < nx_c;  i_c++)
   {
     w_c[k_c]   += w[k];
     k++;
     w_c[k_c]   += 0.5*w[k];
     w_c[k_c+1] += 0.5*w[k];
     k++; k_c++;
   }
   w_c[k_c] += w[k];					/* right column	*/
   k++; k_c++;

   k_c = k_m;
   for (i_c = 0;  i_c < nx_c;  i_c++)
   {
     w_c[k_c]        += 0.5*w[k];
     w_c[k_c+nx_c+1] += 0.5*w[k];
     k++;
     w_c[k_c]	     += 0.25*w[k];
     w_c[k_c+1]      += 0.25*w[k];
     w_c[k_c+nx_c+1] += 0.25*w[k];
     w_c[k_c+nx_c+2] += 0.25*w[k];
     k++; k_c++;
   }
   w_c[k_c]        += 0.5*w[k];
   w_c[k_c+nx_c+1] += 0.5*w[k];				/* right column	*/
   k++; k_c++;
   k_m += nx_c+1;
 } 
 
   k_c = k_m;
   for (i_c = 0;  i_c < nx_c;  i_c++)			/* top row	*/
   {
     w_c[k_c]   += w[k];
     k++;
     w_c[k_c]   += 0.5*w[k];
     w_c[k_c+1] += 0.5*w[k];
     k++; k_c++;
   }
   w_c[k_c] += w[k];					/* top right vertex */

	/* Handle Dirichlet B.C.  = neigh[.] == -1 */
 
 for (l=0; l<=3; l++)
 {
  if (neigh[l]==-1)
    switch (l)
    {
     case 0 : 
     {
       vdcopy(nx_c+1, &w_c[0], 1, &dzero, 0);
       break;
     }
     case 2 : 
     {
       vdcopy(nx_c+1, &w_c[ny_c*(nx_c+1)], 1, &dzero, 0);
       break;
     }
     case 1 : 
     {
       vdcopy(ny_c+1, &w_c[nx_c], nx_c+1, &dzero, 0);
       break;
     }     
     case 3 : 
     {
       vdcopy(ny_c+1, &w_c[0], nx_c+1, &dzero, 0);
       break;
     }     
    }
 }

 return;
}
