#include <iostream>
#include <cmath>
using namespace std;
#include "header.h"

void JacobiSolve(const int nx, const int ny, 
	const double sk[], const int id[], const int ik[], 
	const double f[], double u[] )
{
  const double omega   = 1.0;
  const int    maxiter = 1000;
  const double tol  = 1e-5,				/* tolerance   */
               tol2 = tol*tol;				/* tolerance^2 */
  
  const int nnode = (nx+1)*(ny+1);
  double *dd = new double [nnode],			/* matrix diagonal */
         *r  = new double [nnode],			/* residual */
	 *w  = new double [nnode];			/* correction */
  cout << endl << " Start Jacobi solver for " << nnode << " d.o.f.s"  << endl;
  
  GetDiag(nx, ny, sk, id, dd);                         /*  diag */
	 
  /*  Choose initial guess  */
  const double dzero=0.0;
  vdcopy(nnode, u, 1, &dzero, 0);		/*  u := 0         */

  /*  Initial sweep  */

  vdcopy(nnode, r, 1, f, 1);			/*                 */
  CrsMult(0, nnode-1, r, u, id, ik, sk, -1.0);	/*  r := f - K*u   */
  
  vddiv(nnode, w, 1, r, 1, dd, 1);		/*  w := D^{-1}*r  */
  double sigma0 = dscapr(nnode,w,r);	/* s0 := <w,r>     */
  
  /* Iteration sweeps */
  int iter  = 0;
  double sigma = sigma0;
  while ( sigma > tol2*sigma0 && maxiter>iter)
  {
    ++iter;
    vdaxpy(nnode, u, u, omega, w );		/*  u := u + om*w  */  
    vdcopy(nnode, r, 1, f, 1);			/*                 */
    CrsMult(0, nnode-1, r, u, id, ik, sk, -1.0);/*  r := f - K*u   */

    vddiv(nnode, w, 1, r, 1, dd, 1);		/*  w := D^{-1}*r  */ 
    sigma = dscapr(nnode,w,r);		       /* s0 := <w,r>     */
    
//      	cout << "Iteration " << iter << " : " << sqrt(sigma/sigma0) << endl;
  }
  cout << "aver. Jacobi rate :  " << exp(log(sqrt(sigma/sigma0))/iter) << endl;
  cout << "final error: " << sqrt(sigma/sigma0) << " (rel)   " << sqrt(sigma) << " (abs)\n";
 
  delete [] w; delete [] r; delete [] dd;
  return;
}

/*  initial guess u and accumulated diagonal are INPUT */

void JacobiSolve2(const int nx, const int ny,
	const double sk[], const int id[], const int ik[],
	const double f[], double u[], const double dd[], double aux[] )
{
  const double  omega   = 1.0, tol  = 1e-5, tol2 = tol*tol;
  const int    maxiter=100, nnode = (nx+1)*(ny+1);
  double *r, *w, sigma, sigma0;
  int  iter;

  r = aux;
  w = new double[nnode];

  /*  Initial sweep  */
  vdcopy(nnode, r, 1, f, 1);			/*                 */
  CrsMult(0, nnode-1, r, u, id, ik, sk, -1.0);	/*  r := f - K*u   */

  vddiv(nnode, w, 1, r, 1, dd, 1);		/*  w := D^{-1}*r  */
  sigma0 = dscapr(nnode,w,r);		/* s0 := <w,r>     */

  /* Iteration sweeps */
  iter  = 0;
  sigma = sigma0;
  while ( sigma > tol2*sigma0 && maxiter>iter)
  {
    iter++;
    vdaxpy(nnode, u, u, omega, w );		/*  u := u + om*w  */
    vdcopy(nnode, r, 1, f, 1);			/*                 */
    CrsMult(0, nnode-1, r, u, id, ik, sk, -1.0);/*  r := f - K*u   */

    vddiv(nnode, w, 1, r, 1, dd, 1);		/*  w := D^{-1}*r  */
    sigma = dscapr(nnode,w,r);		/* s0 := <w,r>     */

/*    if (myid==0)
    	printf("Iteration %u : %g\n",iter,sqrt(sigma/sigma0));   */
  }
/*  printf("Process %u : %u Iterations\n",myid,iter); */

  delete [] w;
  return;
}