//		MPI code in C++.
//		See [Gropp/Lusk/Skjellum, "Using MPI", p.33/41 etc.]
//		and  /opt/mpich/include/mpi2c++/comm.h  for details
#include <cstdio>       // printf
#include <iostream>
#include <cmath>
using namespace std;
#include "header.h"
#include "mg.h"

int main(int argc, char* argv[])
{
    const bool bdebug=false;		// debugging at the end??
    const int nlevels = 3; 				/* set number of mg levels */

    const int numprocs = 1;
    const int myrank   = 0;

    if (myrank == 0)
    { cout << "\n There are " << numprocs << " processes running.\n \n"; }

    const int procx = static_cast<int>(sqrt(numprocs+0.0));
    const int procy = procx;

    if (procy * procx != numprocs)
    {
      cout << "\n Wrong number of processors !\n \n";
    }
    else
    {
      int neigh[4], color;
      IniGeom(myrank, procx, procy, neigh, color);

      /*                                              subdomain coordinates */
      double xl,xr,yb,yt;
      IniCoord(myrank, procx, procy, xl, xr, yb, yt);

      bool ScaleUp = !true;
      int nx,ny,NXglob,NYglob;
      /* number of local intervals on (xl,xr)=:nx, (yb,yt)=:ny */
	nx = 2;
	ny = 2;
	NXglob = nx*procx;
	NYglob = ny*procy;

      /* number of nodes per direction (xl,xr)=nx+1=:nex, (yb,yt)=ny+1=:ney */
      const int nnx = nx + 1,
                nny = ny + 1,
      /* number of degrees of freedom  */
                nnode = nnx*nny;
//
      double t_start = clock(), t_mgm;
      int ierr;
      ControlStruct control;
      PointerStruct pointers;
      /* Set mg control parameters */
      control.cycle = 1;		/* V-cycle			*/
      control.npre  = 1;		/* #sweeps in presmoothing	*/
      control.npost = 1;		/* #sweeps in postsmoothing	*/

 // C --> C++
      ierr = AllocLevels(nlevels, nx, ny, &pointers);
      if (ierr!=0) printf("\nError No. %i in AllocLevels.\n",ierr);
      const int n1 = pointers.nx[nlevels-1]+1;
      printf("Unknowns: (%u)² = %d per processor\n",n1,n1*n1);

      ierr = IniLevels(xl, xr, yb, yt, neigh, nlevels, &pointers);
//
      MGMSolver(&pointers, &control, nlevels-1, neigh);

      const int level = nlevels-1;
//
      SaveVectorP(myrank, "t.dat", pointers.u[level], pointers.nx[level],
      				pointers.ny[level], xl, xr, yb, yt);
      if (myrank==0) printf("-----------\n");
//
//       ierr = FreeLevels(nlevels, &pointers);   // GH: Bug in memory alloc/dealloc  !!
      
  }

}


