#include <cmath>
#include <malloc.h>
#include "header.h"
#include "mg.h"

void MGM(const PointerStruct* const pp, const ControlStruct* const crtl,
	 const int level, const int neigh[]
	)
{
 double *sk, *dd, *u, *f, *d, *w, *aux, *d_c, *w_c, dzero=0.0;
 int    *id, *ik, nx, ny, nnode, nx_c, ny_c, j;

 nx = pp->nx[level];
 ny = pp->ny[level];
 sk = pp->sk[level];
 id = pp->id[level];
 ik = pp->ik[level];
 dd = pp->dd[level];
 u  = pp->u[level];
 f  = pp->f[level];
 aux= pp->aux[level];			/* all fixed fine matrices/arrays */
 nnode = (nx+1)*(ny+1);
 d  = aux;				/* defect on fine grid		*/
 w  = aux;				/* correction on fine grid	*/

 if (level==0) 
 { 
  /* if (myid==0) printf("-------------\nCoarse grid solver\n"); */
  JacobiSolve2(nx, ny, sk, id, ik, f, u, dd, aux );
 }
 else
 {

  nx_c = pp->nx[level-1];
  ny_c = pp->ny[level-1]; 
  d_c  = pp->f[level-1];		/* defect on corse grid		*/
  w_c  = pp->u[level-1];		/* correction on coarse grid	*/
  
					/* Presmoothing			*/
  	/* if (myid==0) printf("%u : Smooth-Pre\n",level); 		*/
  JacobiSmooth(nx, ny, sk, id, ik, f, u, dd, aux, crtl->npre );
 
  	/* if (myid==0) printf("%u : Mult\n",level);			*/
  vdcopy(nnode, d, 1, f, 1);		/* d := f - K*u   		*/
  CrsMult(0, nnode-1, d, u, id, ik, sk, -1.0);

  	/* if (myid==0) printf("%u : Restrict\n",level);		*/
  Restrict(nx, ny, d,  nx_c, ny_c, d_c, neigh);
 
  vdcopy((nx_c+1)*(ny_c+1), w_c, 1, &dzero, 0);		/*  w_c := 0	*/

  	/* if (myid==0) printf("%u : MGM-cycle\n",level);		*/
  for (j=1; j<=crtl->cycle; j++)
       MGM(pp, crtl, level-1, neigh);
  
  	/* if (myid==0) printf("%u : Interpolate\n",level);		*/
  Interpolate(nx, ny, w,  nx_c, ny_c, w_c, neigh);

  vdaxpy(nnode, u, u, 1.0, w );		/*  u := u + w			*/  

  
					/* Postsmoothing		*/
  	/* if (myid==0) printf("%u : Smooth-Post\n",level);		*/
  JacobiSmooth(nx, ny, sk, id, ik, f, u, dd, aux, crtl->npost );
 
 }

 return;
}


void MGMSolver(const PointerStruct* const pp, const ControlStruct* const crtl, const int level,
	       const  int neigh[]
)
{
 const double dzero=0.0, tol  = 1e-4, tol2 = tol*tol;
 const int maxiter = 20;
 double *sk, *u, *f, *w, *aux, *r, sigma0, sigma;
 int    *id, *ik, iter;

 				printf("In MGMSolver\n");

 const int nx = pp->nx[level];
 const int ny = pp->ny[level];
 const int nnode = (nx+1)*(ny+1);
 sk = pp->sk[level];
 id = pp->id[level];
 ik = pp->ik[level];
 u  = pp->u[level];
 f  = pp->f[level];
 aux= pp->aux[level];			/* all fixed fine matrices/arrays */
 r  = aux;				
 w  = (double *) malloc(nnode*sizeof(double));
 
 vdcopy(nnode,u,1,&dzero,0);			/*  u := 0	   */
 
 vdcopy(nnode, r, 1, f, 1);			/*                 */
 CrsMult(0, nnode-1, r, u, id, ik, sk, -1.0);	/*  r := f - K*u   */
 
 vdcopy(nnode, w, 1, r, 1);			/*                 */
 sigma0 = dscapr(nnode,w,r);	          	/* s0 := <w,r>     */
 
  /* Iteration sweeps */
 iter  = 0;
 sigma = sigma0;
 while ( sigma > tol2*sigma0 && maxiter>iter)
 {
   iter++;
   
   MGM(pp, crtl, level, neigh);
/*   JacobiSmooth(nx, ny, sk, id, ik, f, u, pp->dd[level], aux, 100 ); */

   vdcopy(nnode, r, 1, f, 1);			/*                 */
   CrsMult(0, nnode-1, r, u, id, ik, sk, -1.0);	/*  r := f - K*u   */

   vdcopy(nnode, w, 1, r, 1);			/*                 */
   sigma = dscapr(nnode,w,r);		/* s0 := <w,r>     */   

       printf("MG-Iteration  %u : %g\n",iter,sqrt(sigma/sigma0));     
 }
 
 	printf("aver. MG rate :  %g\n", exp(log(sqrt(sigma/sigma0))/iter));

 free(w);
 
 return;
}
